#ifndef THINGS_THINGINSTANCEBASE_H
#define THINGS_THINGINSTANCEBASE_H
///
/// @file 
/// @version $Id$
/// @date $Date$
///

#include "ThingSequenceNumber.h"

namespace Things
{

///
/// @brief
/// Abstract base class
///
/// This class is part of the composite pattern where it takes the role
/// of Component.
/// See for example http://en.wikipedia.org/wiki/Composite_pattern
///
/// This class is also part of the visitor pattern where it takes the role
/// of Element.
/// See for example http://en.wikipedia.org/wiki/Visitor_pattern
///
class ThingInstanceBase
{
public:
   /// @name House-keeping
   /// @{

   ThingInstanceBase();

   ///
   /// @throws none
   ///
   virtual ~ThingInstanceBase();

   /// @}
public:
   /// @name Operations
   /// @{

   ///
   /// @brief
   /// Return a sequence number appropriate to the rule or collection of rules.
   ///
   virtual ThingSequenceNumber getThingSequenceNumber() const = 0;

   /// @}
public:
   /// @name Mutators
   /// @{

   ///
   /// @brief
   /// Assign a sequence number to a rule or collection of rules.
   ///
   virtual void setThingSequenceNumber(ThingSequenceNumber sequenceNumber_) = 0;

   /// @}
}; //endclass ThingInstanceBase

} //end namespace

#endif
